import {ApplicationState, FALLBACK_CHARSET, FALLBACK_LANGUAGE, FALLBACK_MEDIA_TYPE, Resource} from '../common';
import {createServer, CreateServerParams, Server} from './servers/http';
import {BackendState} from './common';
import {DataSource} from './data-source';

export interface Backend<T extends DataSource = DataSource> {
	showTotalItemCountOnGetCollection(b?: boolean): this;
	showTotalItemCountOnCreateItem(b?: boolean): this;
	checksSerializersOnDelete(b?: boolean): this;
	throwsErrorOnDeletingNotFound(b?: boolean): this;
	createHttpServer(serverParams?: CreateServerParams): Server;
	dataSource?: (resource: Resource) => T;
}

export interface CreateBackendParams<T extends DataSource = DataSource> {
	app: ApplicationState;
	dataSource: T;
}

export const createBackend = (params: CreateBackendParams) => {
	const backendState: BackendState = {
		app: params.app,
		dataSource: params.dataSource,
		cn: {
			language: FALLBACK_LANGUAGE,
			charset: FALLBACK_CHARSET,
			mediaType: FALLBACK_MEDIA_TYPE,
		},
		showTotalItemCountOnGetCollection: false,
		showTotalItemCountOnCreateItem: false,
		throwsErrorOnDeletingNotFound: false,
		checksSerializersOnDelete: false,
	};

	return {
		showTotalItemCountOnGetCollection(b = true) {
			backendState.showTotalItemCountOnGetCollection = b;
			return this;
		},
		showTotalItemCountOnCreateItem(b = true) {
			backendState.showTotalItemCountOnCreateItem = b;
			return this;
		},
		throwsErrorOnDeletingNotFound(b = true) {
			backendState.throwsErrorOnDeletingNotFound = b;
			return this;
		},
		checksSerializersOnDelete(b = true) {
			backendState.checksSerializersOnDelete = b;
			return this;
		},
		createHttpServer(serverParams = {} as CreateServerParams) {
			return createServer(backendState, serverParams);
		},
	} satisfies Backend;
};
