import {tokenizePath} from './utils';

export const set = (origObject: Record<string, unknown>, path: string, value: unknown) => {
	if (path.length <= 0) {
		return origObject;
	}

	const pathFragments = tokenizePath(path);
	let cursor = origObject;
	let thisPath = pathFragments.shift() as string;
	while (pathFragments.length > 0) {
		cursor = cursor?.[thisPath] as Record<string, unknown>;
		thisPath = pathFragments.shift() as string;
	}

	if (typeof cursor === 'undefined') {
		throw new Error(`Could not set path: ${path}`);
	}

	cursor[thisPath] = value;
	return origObject;
};

export const get = (origObject: Record<string, unknown>, path: string) => {
	if (path.length <= 0) {
		return origObject;
	}
	const pathFragments = tokenizePath(path);
	let cursor = origObject;
	let thisPath = pathFragments.shift() as string;
	while (pathFragments.length > 0) {
		cursor = cursor?.[thisPath] as Record<string, unknown>;
		thisPath = pathFragments.shift() as string;
	}

	return cursor?.[thisPath];
};

export const remove = (origObject: Record<string, unknown>, path: string) => {
	if (path.length <= 0) {
		return origObject;
	}
	const pathFragments = tokenizePath(path);
	let cursor = origObject;
	let thisPath = pathFragments.shift() as string;
	while (pathFragments.length > 0) {
		cursor = cursor?.[thisPath] as Record<string, unknown>;
		thisPath = pathFragments.shift() as string;
	}

	if (typeof cursor === 'undefined') {
		throw new Error(`Could not remove on path: ${path}`);
	}

	delete cursor[thisPath];
	return origObject;
};

export const append = (origObject: Record<string, unknown>, path: string, value: unknown) => {
	if (path.length <= 0) {
		return origObject;
	}
	const pathFragments = tokenizePath(path);
	let cursor = origObject;
	let thisPath = pathFragments.shift() as string;
	while (pathFragments.length > 0) {
		cursor = cursor?.[thisPath] as Record<string, unknown>;
		thisPath = pathFragments.shift() as string;
	}

	if (!Array.isArray(cursor?.[thisPath])) {
		throw new Error(`Could not append on path: ${path}`);
	}

	(cursor[thisPath] as unknown[]).push(value);
	return origObject;
};
